-- Mul8x4.vhd
-- version 2009-01-08 : pipeline gates removed

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library proasic3;
use proasic3.all;

entity Mul8x4 is
  port (
    Clk, WrEn, MulEn : in std_ulogic;
    DataA : in std_ulogic_vector(3 downto 0);
    DataB, Val : in std_ulogic_vector(7 downto 0);
    DataOut : out std_ulogic_vector(11 downto 0)
  );
end Mul8x4;

architecture A3P of Mul8x4 is

  signal LUT1, LUT2 : std_ulogic_vector(7 downto 0);

  component RAM4K9
    generic (MEMORYFILE:string := "");
    port(
      ADDRA11, ADDRA10, ADDRA9,  ADDRA8,  ADDRA7, ADDRA6,
      ADDRA5,  ADDRA4,  ADDRA3,  ADDRA2,  ADDRA1, ADDRA0,
      ADDRB11, ADDRB10, ADDRB9,  ADDRB8,  ADDRB7, ADDRB6,
      ADDRB5,  ADDRB4,  ADDRB3,  ADDRB2,  ADDRB1, ADDRB0,
      DINA8,   DINA7,   DINA6,   DINA5,   DINA4,  DINA3,
      DINA2,   DINA1,   DINA0,   DINB8,   DINB7,  DINB6,
      DINB5,   DINB4,   DINB3,   DINB2,   DINB1,  DINB0,
      WIDTHA0, WIDTHA1, WIDTHB0, WIDTHB1, PIPEA,  PIPEB,
      WMODEA,  WMODEB,  BLKA,    BLKB,    WENA,   WENB,
      CLKA,    CLKB,    RESET : in std_logic;
      DOUTA8, DOUTA7, DOUTA6, DOUTA5, DOUTA4, DOUTA3, 
      DOUTA2, DOUTA1, DOUTA0, DOUTB8, DOUTB7, DOUTB6,
      DOUTB5, DOUTB4, DOUTB3, DOUTB2, DOUTB1, DOUTB0 : out std_logic) ;
  end component;

begin

--------------------------------------------
-- The 4x4x2 Multiply lookup table :
--------------------------------------------

  DualPort512x9 : RAM4K9
  port map(
    ADDRA11 => '0', ADDRA10 => '0',
    ADDRA9  => '0', ADDRA8  => '0',
    ADDRA7  => DataB(3), ADDRA6  => DataB(2),
    ADDRA5  => DataB(1), ADDRA4  => DataB(0),
    ADDRA3  => DataA(3), ADDRA2  => DataA(2),
    ADDRA1  => DataA(1), ADDRA0  => DataA(0),
    DOUTA8 => OPEN,    DOUTA7 => LUT1(7), DOUTA6 => LUT1(6),
    DOUTA5 => LUT1(5), DOUTA4 => LUT1(4), DOUTA3 => LUT1(3),
    DOUTA2 => LUT1(2), DOUTA1 => LUT1(1), DOUTA0 => LUT1(0),
    DINA8 => '0', DINA7 => '0', DINA6 => '0',
    DINA5 => '0', DINA4 => '0', DINA3 => '0',
    DINA2 => '0', DINA1 => '0', DINA0 => '0',
    WIDTHA0 => '1', WIDTHA1 => '1', PIPEA => '0',
    WMODEA =>  '1', WENA => '1', BLKA => MulEn , CLKA => Clk,

    ADDRB11 => '0', ADDRB10 => '0',
    ADDRB9  => '0', ADDRB8  => '0',
    ADDRB7  => DataB(7), ADDRB6  => DataB(6),
    ADDRB5  => DataB(5), ADDRB4  => DataB(4),
    ADDRB3  => DataA(3), ADDRB2  => DataA(2),
    ADDRB1  => DataA(1), ADDRB0  => DataA(0),
    DOUTB8 => OPEN,    DOUTB7 => LUT2(7), DOUTB6 => LUT2(6),
    DOUTB5 => LUT2(5), DOUTB4 => LUT2(4), DOUTB3 => LUT2(3),
    DOUTB2 => LUT2(2), DOUTB1 => LUT2(1), DOUTB0 => LUT2(0),
    DINB8 => '0',    DINB7 => Val(7), DINB6 => Val(6),
    DINB5 => Val(5), DINB4 => Val(4), DINB3 => Val(3),
    DINB2 => Val(2), DINB1 => Val(1), DINB0 => Val(0),
    WIDTHB0 => '1', WIDTHB1 => '1', PIPEB => '0',
    WMODEB =>  '1', BLKB =>  MulEn, WENB => WrEn, CLKB => Clk,

    RESET => '1'
  );

  DataOut(3 downto 0) <= LUT1(3 downto 0);
  DataOut(11 downto 4) <= std_ulogic_vector(unsigned(LUT2)
                              + unsigned("0000" & LUT1(7 downto 4)));

end A3P;
