-- testdiff.vhd
-- Version 20080816 by whygee@f-cpu.org

-- this testbench exercises the add/sub unit with a LFSR

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity testdiff is
  generic (
    YASEP_SIZE : integer := 32; -- 16 or 32
    PIPELINED  : integer := 1   -- pipelined when 1
  );
end testdiff;

architecture toplevel of testdiff is
  signal Sum : std_logic_vector(YASEP_SIZE downto 0);
  signal DataA, DataB, ROP2_OUTt : std_logic_vector(YASEP_SIZE-1 downto 0);
  signal clk, Addsub, R2Ct, ROP2_S1, ROP2_S2, ROP2_X2 : std_logic;
begin

  ROP2_S1 <= '0';
  ROP2_S2 <= '0';
  ROP2_X2 <= '0';

  DUT:
    entity work.AddSub32(ActelProASIC3)
    generic map(
    	PIPELINED  => PIPELINED,
      YASEP_SIZE => YASEP_SIZE
    )
    port map(
      DataA  => DataA,
      DataB  => DataB,
      Addsub => Addsub,
      ROP2_S1 => ROP2_S1,
      ROP2_S2 => ROP2_S2,
      ROP2_X2 => ROP2_X2,
      ROP2_OUT => ROP2_OUTt,
      ROP2_combine => R2Ct,
      clk    => clk,
      Sum    => Sum
    );

  bench: process 
    variable LFSR: std_logic_vector(YASEP_SIZE*2 downto 0);
    variable A, B, result : unsigned(YASEP_SIZE downto 0);
    variable feedback: std_logic;

  begin
    LFSR := (0=>'1', others=>'0');
    clk <= '0';

    loop
      -- update the LFSR :
      if (YASEP_SIZE=16) then
        -- YASEP_SIZE=16 => X^33 + X^10 +1 in Fibonacci configuration
        feedback := LFSR(32) xor LFSR(9);
      else
        -- YASEP_SIZE=32 => X^65 + X^18 +1
        feedback := LFSR(64) xor LFSR(17);
      end if;
      LFSR := LFSR((YASEP_SIZE*2)-1 downto 0) & feedback;  -- shift

      DataA  <= LFSR( YASEP_SIZE-1    downto 0);
      DataB  <= LFSR((YASEP_SIZE*2)-1 downto YASEP_SIZE);
      Addsub <= LFSR( YASEP_SIZE*2);

      -- propagate in the pipeline
      wait for 1 us;
      clk <= '1';
      wait for 1 us;
      clk <= '0';
      wait for 1 us;
      clk <= '1';
      wait for 1 us;
      clk <= '0';
      wait for 1 us;

      A := unsigned('0'    & DataA);
      B := unsigned(Addsub & DataB); -- ASU_ROP2 does not implement 1 XOR in the CDP
      if (Addsub = '0') then
        result := A + B;
      else
        result := A - B;
      end if;

      assert (std_logic_vector(result) = Sum)
        report " addition is broken ! "
        severity failure;

    end loop;

  end process;

end toplevel;
