-- testdiff.vhd
-- Version 20080812 by whygee@f-cpu.org

library ieee;
use ieee.std_logic_1164.all;

entity testdiff is
  generic (
  	TestNr     : integer := 1;  -- From 0 to 1 : the test you want to run
    YASEP_SIZE : integer := 32; -- 16 or 32
    PIPELINED  : integer := 1   -- pipelined when 1
      -- 2x2x2 tests = 8 tests ...
  );
  port(
    DataA, DataB : in std_logic_vector(YASEP_SIZE-1 downto 0);
    Addsub, clk, add_en,
    ROP2_S1, ROP2_S2, ROP2_X2, ASU_ROP2 : in std_logic; -- ROP2 functions
    diff : out std_logic_vector(YASEP_SIZE downto 0));
end testdiff;

architecture toplevel of testdiff is
  signal SumOrig, SumDUT : std_logic_vector(YASEP_SIZE downto 0);
  signal inA, inB, ROP2_OUTo, ROP2_OUTt : std_logic_vector(YASEP_SIZE-1 downto 0);
  signal R2Ct, R2Co, ROP2_S1r, ROP2_S2r, ROP2_X2r : std_logic;
begin

----------------------------------------------------------------
-- This test checks the formal equivalence between the "netlist"
-- version and the behavioural version. The result of synthesis
-- must contain NO logic gate, diff must be = 0 when PIPELINED=0
-- (otherwise, some differences appear because of the latches)
----------------------------------------------------------------
test0: if TestNr=0 generate
  inA <= (others=>'0'); -- prevents a warning because
  inB <= (others=>'0'); -- these variables are not used in this test
  ROP2_S1r <= '0';
  ROP2_S2r <= '0';
  ROP2_X2r <= '0';

  DUT:
    entity work.AddSub32(ActelProASIC3)
    generic map(
    	PIPELINED  => PIPELINED,
      YASEP_SIZE => YASEP_SIZE
    )
    port map(
      DataA  => DataA,
      DataB  => DataB,
      Addsub => Addsub,
      clk    => clk,
      add_en => add_en,
      ROP2_S1 => ROP2_S1,
      ROP2_S2 => ROP2_S2,
      ROP2_X2 => ROP2_X2,
      ROP2_OUT => ROP2_OUTt,
      ROP2_combine => R2Ct,
      Sum    => SumDUT
    );

  Orig:
    entity work.AddSub32(behaviour)
    generic map(
    	PIPELINED  => 0,
      YASEP_SIZE => YASEP_SIZE
    )
    port map(
      DataA  => DataA,
      DataB  => DataB,
      Addsub => Addsub,
      clk    => clk,
      add_en => add_en,
      ROP2_S1 => ROP2_S1,
      ROP2_S2 => ROP2_S2,
      ROP2_X2 => ROP2_X2,
      ROP2_OUT => ROP2_OUTo,
      ROP2_combine => R2Co,
      Sum    => SumOrig
    );

    diff <= (SumDUT xor SumOrig) when ASU_ROP2='0'
       else ((R2Ct & ROP2_OUTt) xor (R2Co & ROP2_OUTo));

  end generate;

----------------------------------------------------------------
-- This test indicates how fast AddSub32 can be clocked.
----------------------------------------------------------------
test1 : if TestNr=1 generate

  ROP2_OUTo <= (others=>'0');  -- prevents a warning because 
  R2Co <= '0';  --  these variables are not used in this test

  speed:
    entity work.AddSub32(ActelProASIC3)
    generic map(
    	PIPELINED  => PIPELINED,
      YASEP_SIZE => YASEP_SIZE
    )
    port map(
      DataA  => inA,
      DataB  => inB,
      Addsub => Addsub,
      clk    => clk,
      add_en => add_en,
      ROP2_S1 => ROP2_S1r,
      ROP2_S2 => ROP2_S2r,
      ROP2_X2 => ROP2_X2r,
      ROP2_OUT => ROP2_OUTt,
      ROP2_combine => R2Ct,
      Sum    => SumDUT
    );

    SumOrig <= SumDUT when ASU_ROP2='0' else (R2Ct & ROP2_OUTt);

    latche : process(clk)
    begin
      if (clk'event and clk='1') then
        inA  <= DataA;
        inB  <= DataB;
        ROP2_S1r <= ROP2_S1;
        ROP2_S2r <= ROP2_S2;
        ROP2_X2r <= ROP2_X2;
        diff <= SumOrig;
      end if;
    end process;

  end generate;

end toplevel;
