-- Mul8x4.vhd

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library proasic3;
use proasic3.all;

entity Mul8x4 is
  generic (
    PIPELINED  : integer := 1; -- 1=pipelined
    YASEP_SIZE : integer := 16 -- specific to this implementation
  );
  port (
    Clk, WrEn, MulEn : in std_logic;
    DataA : in std_logic_vector(3 downto 0);
    DataB, Val : in std_logic_vector(7 downto 0);
    DataOut : out std_logic_vector(11 downto 0)
  );
end Mul8x4;

architecture A3P of Mul8x4 is

  signal ResTmp12 : std_logic_vector(11 downto 0);
  signal DataAr : std_logic_vector(3 downto 0);
  signal DataBr, Valr, LUT1, LUT2 : std_logic_vector(7 downto 0);

  component RAM4K9
    generic (MEMORYFILE:string := "");
    port(
      ADDRA11, ADDRA10, ADDRA9,  ADDRA8,  ADDRA7, ADDRA6,
      ADDRA5,  ADDRA4,  ADDRA3,  ADDRA2,  ADDRA1, ADDRA0,
      ADDRB11, ADDRB10, ADDRB9,  ADDRB8,  ADDRB7, ADDRB6,
      ADDRB5,  ADDRB4,  ADDRB3,  ADDRB2,  ADDRB1, ADDRB0,
      DINA8,   DINA7,   DINA6,   DINA5,   DINA4,  DINA3,
      DINA2,   DINA1,   DINA0,   DINB8,   DINB7,  DINB6,
      DINB5,   DINB4,   DINB3,   DINB2,   DINB1,  DINB0,
      WIDTHA0, WIDTHA1, WIDTHB0, WIDTHB1, PIPEA,  PIPEB,
      WMODEA,  WMODEB,  BLKA,    BLKB,    WENA,   WENB,
      CLKA,    CLKB,    RESET : in std_logic;
      DOUTA8, DOUTA7, DOUTA6, DOUTA5, DOUTA4, DOUTA3, 
      DOUTA2, DOUTA1, DOUTA0, DOUTB8, DOUTB7, DOUTB6,
      DOUTB5, DOUTB4, DOUTB3, DOUTB2, DOUTB1, DOUTB0 : out std_logic) ;
  end component;

begin
    assert (YASEP_SIZE=16)
      report "YASEP_SIZE must be 16"
      severity failure;

--  process(clk, MulEn, DataAr, DataBr, Val)
--  begin
---- pipeline gate #1:
--    if (PIPELINED=0) or (clk'event and clk='1' and MulEn='1') then
      DataAr <= DataA;
      DataBr <= DataB;
      Valr   <= Val;
--    end if;
--  end process;


--------------------------------------------
-- The 4x4x2 Multiply lookup table :
--------------------------------------------

  DualPort512x9 : RAM4K9
  port map(
    ADDRA11 => '0', ADDRA10 => '0', ADDRA9  => '0', ADDRA8  => '0',
    ADDRA7  => DataBr(3), ADDRA6  => DataBr(2),
    ADDRA5  => DataBr(1), ADDRA4  => DataBr(0),
    ADDRA3  => DataAr(3), ADDRA2  => DataAr(2),
    ADDRA1  => DataAr(1), ADDRA0  => DataAr(0),
    DOUTA8 => OPEN,    DOUTA7 => LUT1(7), DOUTA6 => LUT1(6),
    DOUTA5 => LUT1(5), DOUTA4 => LUT1(4), DOUTA3 => LUT1(3),
    DOUTA2 => LUT1(2), DOUTA1 => LUT1(1), DOUTA0 => LUT1(0),
    DINA8 => '0', DINA7 => '0', DINA6 => '0',
    DINA5 => '0', DINA4 => '0', DINA3 => '0',
    DINA2 => '0', DINA1 => '0', DINA0 => '0',
    WIDTHA0 => '1', WIDTHA1 => '1', PIPEA => '0',
    WMODEA =>  '1', WENA => '1', BLKA => MulEn , CLKA => Clk,

    ADDRB11 => '0', ADDRB10 => '0', ADDRB9  => '0', ADDRB8  => '0',
    ADDRB7  => DataBr(7), ADDRB6  => DataBr(6),
    ADDRB5  => DataBr(5), ADDRB4  => DataBr(4),
    ADDRB3  => DataAr(3), ADDRB2  => DataAr(2),
    ADDRB1  => DataAr(1), ADDRB0  => DataAr(0),
    DOUTB8 => OPEN,    DOUTB7 => LUT2(7), DOUTB6 => LUT2(6),
    DOUTB5 => LUT2(5), DOUTB4 => LUT2(4), DOUTB3 => LUT2(3),
    DOUTB2 => LUT2(2), DOUTB1 => LUT2(1), DOUTB0 => LUT2(0),
    DINB8 => '0',    DINB7 => Valr(7), DINB6 => Valr(6),
    DINB5 => Valr(5), DINB4 => Valr(4), DINB3 => Valr(3),
    DINB2 => Valr(2), DINB1 => Valr(1), DINB0 => Valr(0),
    WIDTHB0 => '1', WIDTHB1 => '1', PIPEB => '0',
    WMODEB =>  '1', BLKB =>  MulEn, WENB => WrEn, CLKB => Clk,

    RESET => '1'
  );

  ResTmp12(3 downto 0) <= LUT1(3 downto 0);
  ResTmp12(11 downto 4) <= std_logic_vector(
    unsigned(LUT2) + unsigned("0000" & LUT1(7 downto 4)));

  process(clk, MulEn, ResTmp12)
  begin
-- pipeline gate #4:
    if (PIPELINED=0) or (clk'event and clk='1' and MulEn='1') then
      DataOut <= ResTmp12;
    end if;
  end process;


end A3P;
