-- test_SHL16.vhd

library IEEE;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_textio.all;
library std;
use std.textio.ALL;


entity test_SHL16 is
  generic (
  	PIPELINED  : integer := 0; -- 1=pipelined
    YASEP_SIZE : integer := 16 -- must be 16
  );
end test_SHL16;

-----------------------------------------------------------------

architecture bench of test_SHL16 is

  signal DataA : std_logic_vector(YASEP_SIZE-1 downto 0);
  signal SHAmount : std_logic_vector(3 downto 0);
  signal clk, SHL_en, Rotation, Direction, Arithmetic : std_logic;
  signal ResultSHL : std_logic_vector(YASEP_SIZE-1 downto 0);

begin

  assert (YASEP_SIZE=16)
    report "YASEP_SIZE must be 16"
    severity failure;

  DUT:
    entity work.SHL_16(behaviour)
    generic map(
    	PIPELINED  => PIPELINED,
      YASEP_SIZE => YASEP_SIZE
    )
    port map(
      DataA      => DataA,
      SHAmount   => SHAmount,
      clk        => clk,
      SHL_en     => SHL_en,
      Rotation   => Rotation,
      Direction  => Direction,
      Arithmetic => Arithmetic,

      ResultSHL  => ResultSHL
    );

  process
    type tlouts is array (15 downto 0) of line;
    variable louts : tlouts;

    procedure run16(dir, rot, arith : std_logic) is
    begin
      Rotation <= rot;
      Direction <= dir;
      Arithmetic <= arith;
      for i in 0 to 15 loop
        SHAmount <= std_logic_vector(to_unsigned(i,4));
        wait for 10 ns;
        write(louts(i),ResultSHL);
        write(louts(i),string'("  "));
      end loop;
    end run16;

    procedure run16x5(data : std_logic_vector(YASEP_SIZE-1 downto 0)) is
      variable lout : line ;
    begin
      write(lout,string'("****************************************"));
      writeline(output,lout);
      DataA <= data;
      run16('0', '0', '0');
      run16('1', '0', '0');
      run16('0', '1', '0');
      run16('1', '1', '0');
      run16('0', '0', '1');

      for i in 0 to 15 loop
        writeline(output,louts(i));
      end loop;

    end run16x5;

  begin
    SHL_en <= '1';
    clk <= '0';

    run16x5("0000000000000000");
    run16x5("1111111111111111");
    run16x5("1000000000000000");
    run16x5("0111111111111111");
    run16x5("0011110000111100");
    run16x5("1100001111000011");

    assert (false)
      report "simulation finished OK"
      severity failure;

  end process;



end bench;