-- testdiff.vhd
-- Version 200090101 by whygee@f-cpu.org

library ieee;
use ieee.std_logic_1164.all;

entity testdiff is
  generic (
  	TestNr     : integer := 1;  -- the test you want to run
    YASEP_SIZE : integer := 16;
    PIPELINED  : integer := 1   -- pipelined when 1
  );
  port(
    DataA, DataB : in std_logic_vector(YASEP_SIZE-1 downto 0);
    clk, add_EN, Addsub, ROP2_S1, ROP2_S2, ROP2_X2, ASU_ROP2 : in std_logic; -- ROP2 functions
    diff : out std_logic_vector(YASEP_SIZE downto 0));
end testdiff;

architecture toplevel of testdiff is
  signal SumDUT, SumDUTr, ROP2_OUTr : std_logic_vector(YASEP_SIZE downto 0);
  signal inA, inB, ROP2_OUTt, ROP2_OUTref : std_logic_vector(YASEP_SIZE-1 downto 0);
  signal add_ENr, Addsubt, R2Ct, Addsubr, R2Cr, ROP2_S1r, ROP2_S2r, ROP2_X2r : std_logic;
begin


----------------------------------------------------------------
-- This test indicates if "ActelProASIC3" is "equivalent" to "behaviour".
-- The result of synthesis should be "O"
----------------------------------------------------------------
test0 : if TestNr=0 generate

    assert (PIPELINED=0)
      report "PIPELINED must be 0 for test#0"
      severity failure;

  inA <= (inA'range=>'0'); -- inA, inB and other pipeline gates are unused
  inB <= (inB'range=>'0'); --  because it's not pipelined.
  rop2_outr <= (rop2_outr'range=>'0');
  add_ENr <= '0';
  Addsubt <= '0';
  Addsubr <= '0';
  rop2_s1r <= '0';
  rop2_s2r <= '0';
  rop2_x2r <= '0';

  reference:
    entity work.ASUROP2_16(behaviour)
    generic map(
    	PIPELINED  => PIPELINED,
      YASEP_SIZE => YASEP_SIZE
    )
    port map(
      DataA  => DataA,
      DataB  => DataB,
      Addsub => Addsub,
      clk    => clk,
      add_EN => add_EN,
      ROP2_S1 => ROP2_S1,
      ROP2_S2 => ROP2_S2,
      ROP2_X2 => ROP2_X2,

      ROP2_OUT => ROP2_OUTref,
      ROP2_combine => R2Cr,
      Sum    => SumDUTr
    );

  ProASIC:
    entity work.ASUROP2_16(ActelProASIC3)
    generic map(
    	PIPELINED  => PIPELINED,
      YASEP_SIZE => YASEP_SIZE
    )
    port map(
      DataA  => DataA,
      DataB  => DataB,
      Addsub => Addsub,
      clk    => clk,
      add_EN => add_EN,
      ROP2_S1 => ROP2_S1,
      ROP2_S2 => ROP2_S2,
      ROP2_X2 => ROP2_X2,

      ROP2_OUT => ROP2_OUTt,
      ROP2_combine => R2Ct,
      Sum    => SumDUT
    );

    diff <= SumDUT xor SumDUTr 
     xor (ROP2_OUTref & R2Cr) xor (ROP2_OUTt & R2Ct);


  end generate;


----------------------------------------------------------------
-- This test indicates how fast ASUROP2_16 can be clocked.
----------------------------------------------------------------
test1 : if TestNr=1 generate

  -- remove warnings
  ROP2_OUTref <= (ROP2_OUTref'range=>'0');
  Addsubr <= '0';
  R2Cr <= '0';

  speed:
    entity work.ASUROP2_16(ActelProASIC3)
    generic map(
    	PIPELINED  => PIPELINED,
      YASEP_SIZE => YASEP_SIZE
    )
    port map(
      DataA  => inA,
      DataB  => inB,
      Addsub => Addsubt,
      clk    => clk,
      add_EN => add_ENr,
      ROP2_S1 => ROP2_S1r,
      ROP2_S2 => ROP2_S2r,
      ROP2_X2 => ROP2_X2r,

      ROP2_OUT => ROP2_OUTt,
      ROP2_combine => R2Ct,
      Sum    => SumDUT
    );

    -- pipeline gates
    latche : process(clk)
    begin
      if (clk'event and clk='1') then
        inA  <= DataA;
        inB  <= DataB;
        add_ENr <= add_EN;
        Addsubt <= Addsub;
        ROP2_S1r <= ROP2_S1;
        ROP2_S2r <= ROP2_S2;
        ROP2_X2r <= ROP2_X2;

        ROP2_OUTr <= R2Ct & ROP2_OUTt;
        SumDUTr <= SumDUT;
      end if;
    end process;

    -- some output
    diff <= SumDUTr when ASU_ROP2='0' else ROP2_OUTr;

  end generate;

end toplevel;
