/*
    VSP/vsp_reg.js
    created 2005-06-11 by whygee@f-cpu.org

    This file manages the internal registers :
     - definitions
     - constructor and construction
     - init
     - display
     - history
     - updates
     - cycle

   version 2005-06-15 : changing the FIFO algo again.

*/

const value_history_depth=16; // size of the register's history

// The core's registers:

var register_names=new Array(
  "A0", "D0",
  "A1", "D1",
  "A2", "D2",
  "A3", "D3",
  "A4", "D4",
  "A5", "D5",
  "R0","R1","R2","R3"
);

// The "object" approach :

function ObjRegister(bit_width) {
  this.bit_width = bit_width;
  this.value = null;
  this.id = null;
  this.history = null;
  this.last_cycle_counter=null;
//  this.breakpoint=null;  later ...
}

// Creates all the internal registers once for all during startup :
var registers = new Array();

// source
registers["SCQ"]=new ObjRegister(2);
registers["SA0"]=new ObjRegister(32);
registers["SD0"]=new ObjRegister(32);
registers["SA1"]=new ObjRegister(32);
registers["SD1"]=new ObjRegister(32);
registers["SA2"]=new ObjRegister(32);
registers["SD2"]=new ObjRegister(32);
registers["SA3"]=new ObjRegister(32);
registers["SD3"]=new ObjRegister(32);
registers["SA4"]=new ObjRegister(32);
registers["SD4"]=new ObjRegister(32);
registers["SA5"]=new ObjRegister(32);
registers["SD5"]=new ObjRegister(32);
registers["SR0"]=new ObjRegister(32);
registers["SR1"]=new ObjRegister(32);
registers["SR2"]=new ObjRegister(32);
registers["SR3"]=new ObjRegister(32);

// after stage 1
registers["A2CQ"]=new ObjRegister(32);
registers["D2CQ"]=new ObjRegister(32);
registers["OPCODE"]=new ObjRegister(8);
registers["IMM16"]=new ObjRegister(16);
registers["READY"]=new ObjRegister(1);
registers["Valid"]=new ObjRegister(1);
registers["Error1"]=new ObjRegister(1);
registers["CQ1"]=new ObjRegister(2);
registers["CQOK1"]=new ObjRegister(1);

// after stage 2
registers["SRC1"]=new ObjRegister(32);
registers["SRC2"]=new ObjRegister(32);
registers["UnitSel"]=new ObjRegister(2);
registers["Function"]=new ObjRegister(3);
registers["DEST1"]=new ObjRegister(4);
registers["DESTOK1"]=new ObjRegister(1);
registers["CQM1"]=new ObjRegister(2);
registers["CQM2"]=new ObjRegister(2);
registers["CQSel"]=new ObjRegister(1);
registers["CQOK2"]=new ObjRegister(1);

// after stage 3
registers["DEST2"]=new ObjRegister(4);
registers["DESTOK2"]=new ObjRegister(1);
registers["RESULT"]=new ObjRegister(32);
registers["CQOK2"]=new ObjRegister(1);
registers["CQ3"]=new ObjRegister(2);

// destination
registers["DCQ"]=new ObjRegister(2);
registers["DA0"]=new ObjRegister(32);
registers["DD0"]=new ObjRegister(32);
registers["DA1"]=new ObjRegister(32);
registers["DD1"]=new ObjRegister(32);
registers["DA2"]=new ObjRegister(32);
registers["DD2"]=new ObjRegister(32);
registers["DA3"]=new ObjRegister(32);
registers["DD3"]=new ObjRegister(32);
registers["DA4"]=new ObjRegister(32);
registers["DD4"]=new ObjRegister(32);
registers["DA5"]=new ObjRegister(32);
registers["DD5"]=new ObjRegister(32);
registers["DR0"]=new ObjRegister(32);
registers["DR1"]=new ObjRegister(32);
registers["DR2"]=new ObjRegister(32);
registers["DR3"]=new ObjRegister(32);

function reset_registers() {
  for (var i in registers) {
    with (registers[i]) {
      value = null;
      history = new histo_FIFO(value_history_depth);
      last_cycle_counter = null;
    }
  }
  registers["SCQ"].value=0;
  registers["SA0"].value=0;
}

function display_register(reg) {
  with (registers[reg]) {
    id.innerHTML = int2hex(value);
  }
}

function display_all_registers() {
  for (var i in registers) {
    with (registers[i]) {
      id.innerHTML = int2hex(value);
    }
  }
}

// When finishing a cycle, there is a copy of the dest to the src regs.
const registers_from=new Array(
  "DCQ",
  "DA0", "DD0", "DA1", "DD1", "DA2", "DD2",
  "DA3", "DD3", "DA4", "DD4", "DA5", "DD5",
  "DR0", "DR1", "DR2", "DR3"
);

const registers_to=new Array(
  "SCQ",
  "SA0", "SD0", "SA1", "SD1", "SA2", "SD2",
  "SA3", "SD3", "SA4", "SD4", "SA5", "SD5",
  "SR0", "SR1", "SR2", "SR3"
);

function cycle_registers() {
  for (var i in registers_from) {
    update_register(registers_to[i],registers[registers_from[i]].value);
  }
}

///////////////////////////////////////////////////////////////////////
// fonctions that also deal with the history

function convert_cycle_value(cycle, value) {
  var t = cycle + '';
  var u;

  // smart algo that adds some spaces for alignment.
  if (t.length < 5)
    t = '    ' + t;
  if (t.length < 7)
    t = '  ' + t;
  if (t.length < 8)
    t = ' ' + t;

  u = int2hex(value);
  if (u.length < 5)
    u = '    ' + u;
  if (u.length < 7)
    u = '  ' + u;
  if (u.length < 8)
    u = ' ' + u;

  return t + ': ' + u;
}

function histo_FIFO(size) { // constructor
  this.length = 0;
  this.index = 0;
  this.max_index = size; // index==0 is never used ...

  this.add_element = function(new_cycle, new_value) {
    this.index++;
    if (this.index > this.max_index)
      this.index = 1;
    if (this.length < this.max_index)
      this.length++;
    this['cycle'+this.index] = new_cycle; // ugly :-)
    this['value'+this.index] = new_value;
  };

  this.display_all = function() {
    var output_string='';
    if (this.length == 0)
      output_string='(empty)';
    else {
      for (var i = this.index; i>0; i--)
        output_string += convert_cycle_value(this['cycle'+i], this['value'+i])+'<br>';
      if (this.length == this.max_index) {
        for (var i = this.max_index; i > this.index; i--)
          output_string += convert_cycle_value(this['cycle'+i], this['value'+i])+'<br>';
      }
    }
    return output_string;
  };
}

function display_history() {
  if (current_id) { // if there is a focus  (is this test necessary here ?)
    with (registers[current_id]) {
      histo_id.innerHTML = history.display_all();
    }
  }
}

function update_register(reg_id, new_value) {
 // reg_id: string du nom, value: int

  with (registers[reg_id]) {
    if (new_value == value)
      return; // sparing some computations

  // verify the register's size
    if ((bit_width < 32) && (new_value >> bit_width)) {
      add_message('red',"Assignation of a number that is too large");
          // missing here : more debug info
      return;
    }

  // a useful safeguard for later
    if (last_cycle_counter == cycle_number) {
      add_message('orange', "Warning : Assignations to "+reg_id+" at the same clock cycle");
    }

  // history update
    if (last_cycle_counter != null){ // if there is at least something in the history
      history.add_element(last_cycle_counter, value);
      if (reg_id == current_id)
        display_history(); // update if focus
    } 

    // last but not least, assignation :
    id.innerHTML = int2hex(new_value);
    value = new_value;
    last_cycle_counter = cycle_number;
  }
}

