/*
    VSP/vspsim.js
    created Sat Mar  5 02:42:06 EST 2005 by whygee@f-cpu.org
    version Sat Mar  5 17:58:36 EST 2005
    version Wed Mar  9 19:55:09 EST 2005 : objectification + cleanup
    version Wed Jun  8 19:09:15 EDT 2005
    version 2005-06-11
    version 2005-06-17 : added a generic hide/show of <div>s
    version 2005-06-19 : cleanup + run/step/stop double callback machinery
    version 2005-07-29 : cleanup: memory
*/

///////////////////////////////////////////////////////////////////////

var version_date="2005-07-29";

// classical, global variables

// for the focus & register edition
var current_id = null; // the name of the focused register
var current_address = null; // in BYTES
var selected_type = null; // The type of the selected item
var bitwidth = 0;

var button = -1; // 0: left, 1: middle, 2: right

// used to access graphic elements
var selected_box = null;  // the selected register's box
var messages = null;      // box of the message logger
var value_area = null;    // the number input area
var cycle_number_id=null; // contains the cycle counter field
var histo_id = null;      // box for a register's values history
var select_id = null;     // shows the name of the currently selected box
var bitwidth_id = null;   //
var run_button_id = null;
var span_addres_id = null;

///////////////////////////////////////////////////////////////////////

function update_cycle_counter(new_counter) {
  cycle_number = new_counter;
  cycle_number_id.innerHTML = cycle_number + '';
}

///////////////////////////////////////////////////////////////////////

var visibilities = new Object(); // a sort of "database" storing div names and sizes

function toggle_div(div_name) {
  var a = document.getElementById(div_name);
  if (a) {
    // DOM style :
    if (a.style.visibility == 'hidden') {
      // restoration
      a.style.width=visibilities['item_width_'+div_name];
      a.style.height=visibilities['item_height_'+div_name];
      a.style.visibility = 'visible';
    }
    else {
      // backup
      visibilities['item_width_'+div_name] = a.style.width;
      visibilities['item_height_'+div_name] = a.style.height;
      a.style.width=0;
      a.style.height=0;
      a.style.visibility = 'hidden';
    }
  }
  else
    add_message('red',div_name + ' not found :-/');
}

///////////////////////////////////////////////////////////////////////


function writeback_value() {
    // max_value/bitwidth and current_id are already set by select_value()
  var a = 0;
  var b;
  var i = 0;
  var t = value_area.value;

  if (selected_box != null){
    if (t.charAt(t.length-1) == 'h') {
// hex2int conversion :
      while (i < (t.length - 1)) {
        b=int_array[t.charAt(i)];
        if (typeof b != 'number') {
          add_message('orange', 'Error: wrong number format');
          return;
        }
        a = (a << 4) + b;
        i++;
      }
    }
    else {
// dec2int conversion :
      while (i < (t.length)) {
        b = t.charCodeAt(i);
        if ((b < 0x30) || (b > 0x39)) {
          add_message('orange', 'Error : wrong number format');
          return;
        }
        a = (a * 10) + (b - 0x30);
        i++;
      }
    }

    if ((bitwidth < 32) && (a >> bitwidth)) {
      add_message('orange', 'Error : '+current_id+' must fit in '+bitwidth+' bits !');
      return;
    }

    switch (current_type) {
      case 1:
        update_register(current_id, a);
        break;

      case 2:
        update_memory(current_address, a);
        break;

/*      case 3:
        update_sr(current_id, a);
        break; */

      default:
        add_message('red', 'unknown data type for update');
    }
  }
}

function blur_item() {
  selected_box.style.border = '0'; // I have tried with '' but this doesn't work with Moz 1.4
  span_addres_id.innerHTML = '';
  current_id=null;
  current_type=null;
}

function select_value (me, address) {
// me = plain name of the selected register
// OR id of the memory cell

// preliminary backup
  value_area.focus();

  if (current_id != me) {
// "blur" the previous, different item
    if (selected_box) // but if it is not set, there is no previous focus.
      blur_item();

    if (address != null) {
//    add_message('green', 'address '+(address*4)+' selected');
      current_type=2;
      current_address=address<<2;
      span_addres_id.innerHTML = 'selected address : '+ int2hex(current_address) +'h';
    }
    else {
      current_type=1; // register
      span_addres_id.innerHTML = '';
    }

// update of the global variables
    current_id = me;
    selected_box = document.getElementById(me);

// memory words are all 32 bits wide
    if (address != null)
      bitwidth = 32;
    else
      bitwidth = registers[me].bit_width;

// update of the edition/input box :
    select_id.innerHTML=me;
    bitwidth_id.innerHTML=bitwidth+'';

// highlight/focus of the register
    selected_box.style.border='thin solid black';
  }

  if (button != 1) { // middle button -> paste, so no update of the input box
    if (selected_box.innerHTML != 'u')
      value_area.value = selected_box.innerHTML + 'h';
    else
      value_area.value = '0';
  }
  if (address == null)
    display_history(); // warning : the register's name is passed by a global variable
}

// called from "update_form" in the HTML part, it adds focus (more handy)
function update_value () {
  if (current_id) {
    writeback_value();
    value_area.focus();
  }
  else
    add_message('orange','no item selected');
}

// this function is fired before the "onclick" is executed,
// so it stores the button type before "onclick" needs it.
function get_button(e) {
  button = e.button;

// is this part executed 2x ?
  if ((button == 1) && (registers[e.target.id].id)) { // does e.target.id exist in the id table ?
    select_value (e.target.id);
    writeback_value();
    e.stopPropagation(); // cancel propagation to the rest (might spare some CPU time)
  }
}

///////////////////////////////////////////////////////////////////////

function step() {
  update_cycle_counter(cycle_number + 1);
  cycle_memory(); // BEFORE cycle_registers() !
  cycle_registers();
  // mise  jour de la fentre d'dition
  if (current_id) {
    if (selected_box.innerHTML != 'u')
      value_area.value = selected_box.innerHTML + 'h';
    else
      value_area.value = '0';
  }
}

var stop_req=false; // set to true by stop_run(), reset to false by run_body()
var running=false; // state of the simulator. when true, nothing else should be done.
var run_timeout;

function run_body() { // called back from start_run()
  step();
  if (stop_req==false)
    run_timeout=window.setTimeout("run_body()",1);
  else {
    running=false;
    stop_req=false;
  }
}

// uuuarghhh ugly :-/
var action=new Function;
var wait_call_back;

function call_back_action(){
  if (running==false) {
    run_button_id.innerHTML="&nbsp;RUN&nbsp;";
    action();
  }
  else
    wait_call_back=window.setTimeout("call_back_action();",1); // else try later...
}

function stop_run() {
  if (running==true) {
    stop_req=true;
    window.clearTimeout(run_timeout); // This might happen while step() is executing
    window.clearTimeout(wait_call_back); // you never know
  }
}

// some hooks, directly from the HTML document
// They have to wait for the run to finish to be reliable.
// But there is no way to wait for the current step to finish except the following :-/

function button_run() {
  if (resetting==false) {
    if (running==true) {
      stop_run();
      action= function () {
//
      }
      call_back_action();
    }
    else {
      run_button_id.innerHTML="&nbsp;STOP&nbsp;";
      running=true;
      stop_req=false;
      run_body();
      timeout=window.setTimeout("run_body()",1);
    }
  }
}

var resetting=false;

function button_reset() {
  if (resetting==false) {
    resetting=true;
    stop_run();
    action=reset_simulator;
    call_back_action();
  }
}

function button_step() {
  if (resetting==false) {
    stop_run();
    action=step;
    call_back_action();
  }
}

function button_update() {
  if ((resetting==false)&&(running==false)) {
    stop_run();
    action=update_value;
    call_back_action();
  }
//  else
//    add_message('red', 'button_update() not executed'+resetting+running);
}

///////////////////////////////////////////////////////////////////////

function reset_simulator (){
  var u=document.getElementById("reset");
  u.innerHTML="&nbsp;.&nbsp;.&nbsp;.&nbsp;.&nbsp;.&nbsp;.&nbsp;.&nbsp;.&nbsp;.&nbsp;.&nbsp;";
  resetting=true;
  window.setTimeout('rest_of_reset_simulator();',1); // force display refresh
}

function rest_of_reset_simulator (){
  if (current_id)
    blur_item();
  update_cycle_counter(0);
  reset_messages();
  value_area.value = '0'; // clear the input box
  reset_registers();
  display_all_registers();
  reset_memory();
  window.setTimeout('end_of_reset_simulator();',1); // force display refresh and other things
}

function end_of_reset_simulator() {
  display_history();
  add_message('blue', '<a href="http://f-cpu.seul.org/whygee/VSP/vspsim.html">http://f-cpu.seul.org/whygee/VSP/vspsim.</a> version ' + version_date);
  add_message('green', 'Simulator is reset to startup state.');
  add_message('green', 'Select one item to edit');
  var u=document.getElementById("reset");
  u.innerHTML="&nbsp;Reset the simulator&nbsp;";
  resetting=false;
}

// init of the pointers to different zones
// to avoid looking up every time
function init_page() {
  var i;
  messages = document.getElementById('messages');
  messages.innerHTML="onload=init_page(); going on ..........."
  value_area = document.update_form.value_area;
  debug = document.getElementById('status');
  cycle_number_id = document.getElementById('cycle_number');
  histo_id = document.getElementById('histo_id');
  select_id = document.getElementById('select_id');
  bitwidth_id = document.getElementById('bitwidth_id');
  run_button_id = document.getElementById('run');
  span_addres_id = document.getElementById('span_address');

  // This one can be done only when the page is completely loaded and parsed :
  // (this is also why 'registers' is not reset by a flush+reallocation)
  for (i in registers) {
    registers[i].id = document.getElementById(i);
  }

//  for (i=0; i<mem_size; i++) {
//    memory_id[i]= document.getElementById('mem'+i);
//  }

 // catch the mouse events so we can read the button number
  document.addEventListener("click",get_button,true);

  reset_simulator();
}

///////////////////////////////////////////////////////////////////////
