/*
file: mksources.js
created by Yann GUIDON 2007-09-05
*/

/* an empty function that embeds comments in the compiled code */
function COMMENT() {}

/* The JS2C translation code */

/* recognized type prefixes : */
var type_in_prefixes =new Array( "void_", "unsigned_", "signed_", "long_", "int_", "U32_", "S32_", "char_", "ptr_" );
var type_out_prefixes=new Array( "void",  "unsigned",  "signed",  "long",  "int",  "U32",  "S32",  "char",  "*" );
var type_translation =new Array();
var type_in_regexes  = new Array();
const regex_all_prefixes_string='('+type_in_prefixes.join('|')+')[a-zA-Z_][a-zA-Z_0-9]*';
const regex_all_prefixes=new RegExp(regex_all_prefixes_string);
for (i=0; i<type_in_prefixes.length; i++) {
  type_in_regexes[i]=new RegExp(type_in_prefixes[i],'g');
  type_translation[type_in_prefixes[i]]=type_out_prefixes[i];
}
const regnospace=new RegExp('[ ]','g');

var debug_string='';

/* list of the variables */
var varlist_init, varlist_name, varlist_type, var_number=0;

/* dump the variable's table */
function print_variables() {
  s='';
  for (i=0; i<var_number; i++) {
    s += varlist_init[i]+' : '+varlist_type[i]+varlist_name[i]+'\n';
  }
  return s;
}

/* return a prefix, if one is found */
function scan_prefix(name) {
  for (var i in type_in_prefixes) {
    j=type_in_prefixes[i];
    if (name.indexOf(j)==0) {
      return j;
    }
  }
  return '';
}

/* get a string (the JS variable name),
   analyzes it, registers it and returns its type+name */
function register_variable(init_name) {
  new_type='';
  new_name=init_name;

  var i=scan_prefix(new_name); // init the first round
  while (i!='') {
    new_type+=type_translation[i]+' '; // translation of the type prefix
    new_name=new_name.slice(i.length);
    i=scan_prefix(new_name); // prepare another round
  }

  /* test the unicity of the input and output */
  for (var i in varlist_name)
    if (varlist_name[i]==new_name)
      return "#error "+new_name+" is already declared";

  varlist_init[var_number]=init_name;
  varlist_name[var_number]=new_name;
  varlist_type[var_number]=new_type;
  var_number++;
  return new_type+new_name;
}

/* the main JS2C routine accepts a function as input */
function show_code(input_function) {

  /* (re-)init the variables' list */
  varlist_init = new Array();
  varlist_type = new Array();
  varlist_name = new Array();
  var_number=0;

  s=input_function.toString();

  /* remove the leading "function " */
  s=s.slice(9);  

  /* function name */
  i=s.indexOf('(');
  funcname=s.slice(0,i); // there should be no space left here
  s=s.slice(i+1);
  funcname=register_variable(funcname);

  /* argument list */
  i=s.indexOf(')');
  arglist=s.slice(0,i);
  s=s.slice(i+1);
  arglist=arglist.replace(regnospace,'');
  arguments=arglist.split(',');
  for (i in arguments) {
    arguments[i]=register_variable(arguments[i]);
  }

  // Replaces the comments :

  // replace the existing variables in the rest of the text
  for (i=0; i<var_number; i++) {
    rx=new RegExp(varlist_init[i],'g');
    s=s.replace(rx,varlist_name[i]);
  }

  // Scan the rest of the body
  while (s.search(regex_all_prefixes)>0) {
    // we have found a prefix \o/
    // so now let's extract the variable's name 
    initname=s.match(regex_all_prefixes)[0];

    // then replace it with the new definition,
    newname=register_variable(initname);
    s=s.replace(initname,newname);

    // and finally, replace all the remaining identical variable names in the rest of the body.
    rx=new RegExp(initname,'g');
    s=s.replace(rx,varlist_name[var_number-1]);
  }
  return debug_string+'\n\n**\n\n'+ funcname+" ( "+arguments.join(', ')+" )\n"+s+"\n\nList of the variables :\n"+print_variables();
}
